@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-file-helpers.c,v 1.34 2005/11/10 21:32:32 crispin Exp $
+ *  $Id: ephy-file-helpers.c,v 1.36 2005/12/11 17:44:10 chpe Exp $
  */
 
 #include "config.h"
@@ -309,13 +309,13 @@ ephy_ensure_dir_exists (const char *dir)
 	{
 		if (g_file_test (dir, G_FILE_TEST_EXISTS) == TRUE)
 		{
-			g_warning (_("%s exists, please move it out of the way."), dir);
+			g_warning (_("\"%s\" exists, please move it out of the way."), dir);
 			return FALSE;
 		}
 
 		if (mkdir (dir, 488) != 0)
 		{
-			g_warning (_("Failed to create directory %s."), dir);
+			g_warning (_("Failed to create directory \"%s\"."), dir);
 			return FALSE;
 		}
 	}
